@@ -1,5 +1,3 @@
-
-
 /* 
  * Library for Posix async read operations with hints.
  * Author: Don Capps
@@ -96,7 +94,16 @@
 
 #include <sys/types.h>
 #include <aio.h>
-#if defined(solaris) || defined(linux) || defined(SCO_Unixware_gcc)
+
+#if defined(_LARGEFILE64_SOURCE) && !defined(__LP64__)
+#	define aio_error	aio_error64
+#	define aio_return	aio_return64
+#	define aio_read 	aio_read64
+#	define aio_cancel	aio_cancel64
+#	define aio_write	aio_write64
+#endif
+
+#if defined(solaris) || defined(linux) || defined(SCO_Unixware_gcc) || defined(__NetBSD__)
 #else
 #include <sys/timers.h>
 #endif
@@ -119,7 +126,7 @@
 #include <stdlib.h>
 #endif
 
-#if (defined(solaris) && defined(__LP64__)) || defined(__s390x__) || defined(FreeBSD)
+#if (defined(solaris) && defined(__LP64__)) || defined(__s390x__) || defined(__FreeBSD__) || defined(__NetBSD__)
 /* If we are building for 64-bit Solaris, all functions that return pointers
  * must be declared before they are used; otherwise the compiler will assume
  * that they return ints and the top 32 bits of the pointer will be lost,
@@ -132,11 +139,15 @@
 #include <strings.h> /* For the BSD string functions */
 #endif
 
-void mbcopy(char *source, char *dest, size_t len);
+static void mbcopy(const char *source, char *dest, size_t len);
 
 
 #if !defined(solaris) && !defined(off64_t) && !defined(_OFF64_T) && !defined(__off64_t_defined) && !defined(SCO_Unixware_gcc)
+#	if defined(bsd4_4)
+typedef off_t off64_t;
+#	else
 typedef long long off64_t;
+#	endif
 #endif
 #if defined(OSFV5)
 #include <string.h>
@@ -150,15 +161,14 @@ extern int one;
  * cache, pointed to by async_init(gc) will be of
  * this structure type.
  */
-char version[] = "Libasync Version $Revision: 3.24 $";
+static const char version[] = "Libasync Version $Revision: 3.34 $";
 struct cache_ent {
-	struct aiocb myaiocb;			/* For use in small file mode */
-#ifdef _LARGEFILE64_SOURCE 
-#if defined(__CrayX1__)
-	aiocb64_t myaiocb64;		/* For use in large file mode */
+#if defined(_LARGEFILE64_SOURCE) && defined(__CrayX1__)
+	aiocb64_t myaiocb;		/* For use in large file mode */
+#elif defined(_LARGEFILE64_SOURCE) && !defined(__LP64__)
+	struct aiocb64 myaiocb;		/* For use in large file mode */
 #else
-	struct aiocb64 myaiocb64;		/* For use in large file mode */
-#endif 
+	struct aiocb myaiocb;
 #endif 
 	long long fd;				/* File descriptor */
 	long long size;				/* Size of the transfer */
@@ -191,22 +201,41 @@ struct cache {
 
 long long max_depth;
 extern int errno;
-struct cache_ent *alloc_cache();
-struct cache_ent *incache();
+static struct cache_ent *alloc_cache();
+static struct cache_ent *incache();
+
+#ifdef HAVE_ANSIC_C
+void async_init(struct cache **,int, int);
+int async_suspend(struct cache_ent *);
+void end_async(struct cache *);
+void takeoff_cache(struct cache *, struct cache_ent *);
+void del_cache(struct cache *);
+void putoninuse(struct cache *,struct cache_ent *);
+void takeoffinuse(struct cache *);
+struct cache_ent * allocate_write_buffer( struct cache *, long long , long long ,long long, long long, long long, long long, char *, char *);
+void async_put_on_write_queue(struct cache *, struct cache_ent *);
+void async_write_finish(struct cache *);
+void async_wait_for_write(struct cache *);
+int async_read(struct cache *, long long , char *, off64_t, long long, long long, off64_t, long long);
+struct cache_ent * alloc_cache(struct cache *gc,long long fd,off64_t offset,long long size,long long op);
+struct cache_ent * incache(struct cache *, long long, off64_t, long long);
+int async_read_no_copy(struct cache *, long long, char **, off64_t, long long, long long, off64_t, long long);
+void async_release(struct cache *gc);
+size_t async_write(struct cache *,long long, char *, long long, off64_t, long long);
+size_t async_write_no_copy(struct cache *gc,long long fd,char *buffer,long long size,long long offset,long long depth,char *free_addr);
+#else
 void async_init();
 void end_async();
 int async_suspend();
 int async_read();
-void takeoff_cache();
-void del_cache();
 void async_release();
-void putoninuse();
-void takeoffinuse();
 struct cache_ent *allocate_write_buffer();
 size_t async_write();
 void async_wait_for_write();
 void async_put_on_write_queue();
 void async_write_finish();
+struct cache_ent * alloc_cache();
+#endif
 
 /* On Solaris _LP64 will be defined by <sys/types.h> if we're compiling
  * as a 64-bit binary.  Make sure that __LP64__ gets defined in this case,
@@ -223,11 +252,15 @@ void async_write_finish();
 /***********************************************/
 /* Initialization routine to setup the library */
 /***********************************************/
+#ifdef HAVE_ANSIC_C
+void async_init(struct cache **gc,int fd,int flag)
+#else
 void
 async_init(gc,fd,flag)
 struct cache **gc;
 int fd;
 int flag;
+#endif
 {
 #ifdef VXFS
 	if(flag)
@@ -255,26 +288,40 @@ int flag;
 /***********************************************/
 /* Tear down routine to shutdown the library   */
 /***********************************************/
+#ifdef HAVE_ANSIC_C
+void end_async(struct cache *gc)
+#else
 void
 end_async(gc)
 struct cache *gc;
+#endif
 {
 	del_cache(gc);
-	async_write_finish(gc);
-	free((void *)gc);
+	if(gc && (gc->w_head !=NULL))
+	   async_write_finish(gc);
+
+	if(gc != NULL)
+	   free((void *)gc);
+	gc = NULL;
 }
 
 /***********************************************/
 /* Wait for a request to finish                */
 /***********************************************/
+#ifdef HAVE_ANSIC_C
 int
 async_suspend(struct cache_ent *ce)
+#else
+int
+async_suspend(ce)
+struct cache_ent *ce;
+#endif
 {
 #ifdef _LARGEFILE64_SOURCE 
 #ifdef __LP64__
 	const struct aiocb * const cblist[1] = {&ce->myaiocb};
 #else
-	const struct aiocb64 * const cblist[1] = {&ce->myaiocb64};
+	const struct aiocb64 * const cblist[1] = {&ce->myaiocb};
 #endif
 #else
 	const struct aiocb * const cblist[1] = {&ce->myaiocb};
@@ -320,6 +367,10 @@ async_suspend(struct cache_ent *ce)
  * then the code will wait for the manditory first read.
  *************************************************************************/
 
+#ifdef HAVE_ANSIC_C
+int async_read(struct cache *gc, long long fd, char *ubuffer, off64_t offset, 
+	long long size, long long stride, off64_t max, long long depth)
+#else
 int 
 async_read(gc, fd, ubuffer, offset, size, stride, max, depth)
 struct cache *gc;
@@ -330,6 +381,7 @@ long long size;
 long long stride;
 off64_t max;
 long long depth;
+#endif
 {
 	off64_t a_offset,r_offset;
 	long long a_size;
@@ -347,89 +399,28 @@ long long depth;
 	 */
 	if((ce=(struct cache_ent *)incache(gc,fd,offset,size)))
 	{
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
 		{
 			async_suspend(ce);
 		}
-#else
-		while((ret=aio_error64(&ce->myaiocb64))== EINPROGRESS)
-		{
-			async_suspend(ce);
-		}
-#endif
-#else
-		while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
-		{
-			async_suspend(ce);
-		}
-#endif
 		if(ret)
 		{
-			printf("aio_error 1: ret %d %d\n",ret,errno);
+			printf("aio_error 1: ret %zd %d\n",ret,errno);
 		}
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		retval=aio_return(&ce->myaiocb);
-#else
-#if defined(__CrayX1__)
-		retval=aio_return64((aiocb64_t *)&ce->myaiocb64);
-#else
-		retval=aio_return64((struct aiocb64 *)&ce->myaiocb64);
-#endif
-
-#endif
-#else
-		retval=aio_return(&ce->myaiocb);
-#endif
 		if(retval > 0)
 		{
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 			mbcopy((char *)ce->myaiocb.aio_buf,(char *)ubuffer,(size_t)retval);
-#else
-			mbcopy((char *)ce->myaiocb64.aio_buf,(char *)ubuffer,(size_t)retval);
-#endif
-#else
-			mbcopy((char *)ce->myaiocb.aio_buf,(char *)ubuffer,(size_t)retval);
-#endif
 		}
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		if(retval < ce->myaiocb.aio_nbytes)
-#else
-		if(retval < ce->myaiocb64.aio_nbytes)
-#endif
-#else
-		if(retval < ce->myaiocb.aio_nbytes)
-#endif
 		{
-			printf("aio_return error1: ret %d %d\n",retval,errno);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-			printf("aio_return error1: fd %d offset %ld buffer %lx size %d Opcode %d\n",
+			printf("aio_return error1: ret %zd %d\n",retval,errno);
+			printf("aio_return error1: fd %d offset %lld buffer %p size %zd Opcode %d\n",
 				ce->myaiocb.aio_fildes,
-				ce->myaiocb.aio_offset,
-				(long)(ce->myaiocb.aio_buf),
+				(long long)ce->myaiocb.aio_offset,
+				ce->myaiocb.aio_buf,
 				ce->myaiocb.aio_nbytes,
 				ce->myaiocb.aio_lio_opcode
-#else
-			printf("aio_return error1: fd %d offset %lld buffer %lx size %d Opcode %d\n",
-				ce->myaiocb64.aio_fildes,
-				ce->myaiocb64.aio_offset,
-				(long)(ce->myaiocb64.aio_buf),
-				ce->myaiocb64.aio_nbytes,
-				ce->myaiocb64.aio_lio_opcode
-#endif
-#else
-			printf("aio_return error1: fd %d offset %d buffer %lx size %d Opcode %d\n",
-				ce->myaiocb.aio_fildes,
-				ce->myaiocb.aio_offset,
-				(long)(ce->myaiocb.aio_buf),
-				ce->myaiocb.aio_nbytes,
-				ce->myaiocb.aio_lio_opcode
-#endif
 				);
 		}
 		ce->direct=0;
@@ -443,21 +434,13 @@ long long depth;
 		del_read++;
 		first_ce=alloc_cache(gc,fd,offset,size,(long long)LIO_READ);
 again:
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		ret=aio_read(&first_ce->myaiocb);
-#else
-		ret=aio_read64(&first_ce->myaiocb64);
-#endif
-#else
-		ret=aio_read(&first_ce->myaiocb);
-#endif
 		if(ret!=0)
 		{
 			if(errno==EAGAIN)
 				goto again;
 			else
-				printf("error returned from aio_read(). Ret %d errno %d\n",ret,errno);
+				printf("error returned from aio_read(). Ret %zd errno %d\n",ret,errno);
 		}
 	}
 	if(stride==0)	 /* User does not want read-ahead */
@@ -482,15 +465,7 @@ again:
 		if((ce=incache(gc,fd,r_offset,a_size)))
 			continue;
 		ce=alloc_cache(gc,fd,r_offset,a_size,(long long)LIO_READ);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		ret=aio_read(&ce->myaiocb);
-#else
-		ret=aio_read64(&ce->myaiocb64);
-#endif
-#else
-		ret=aio_read(&ce->myaiocb);
-#endif
 		if(ret!=0)
 		{
 			takeoff_cache(gc,ce);
@@ -500,83 +475,27 @@ again:
 out:
 	if(del_read)	/* Wait for the first read to complete */
 	{
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		while((ret=aio_error(&first_ce->myaiocb))== EINPROGRESS)
 		{
 			async_suspend(first_ce);
 		}
-#else
-		while((ret=aio_error64(&first_ce->myaiocb64))== EINPROGRESS)
-		{
-			async_suspend(first_ce);
-		}
-#endif
-#else
-		while((ret=aio_error(&first_ce->myaiocb))== EINPROGRESS)
-		{
-			async_suspend(first_ce);
-		}
-#endif
 		if(ret)
-			printf("aio_error 2: ret %d %d\n",ret,errno);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
+			printf("aio_error 2: ret %zd %d\n",ret,errno);
 		retval=aio_return(&first_ce->myaiocb);
-#else
-		retval=aio_return64(&first_ce->myaiocb64);
-#endif
-#else
-		retval=aio_return(&first_ce->myaiocb);
-#endif
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		if(retval < first_ce->myaiocb.aio_nbytes)
-#else
-		if(retval < first_ce->myaiocb64.aio_nbytes)
-#endif
-#else
-		if(retval < first_ce->myaiocb.aio_nbytes)
-#endif
 		{
-			printf("aio_return error2: ret %d %d\n",retval,errno);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-			printf("aio_return error2: fd %d offset %lld buffer %lx size %d Opcode %d\n",
+			printf("aio_return error2: ret %zd %d\n",retval,errno);
+			printf("aio_return error2: fd %d offset %lld buffer %p size %zd Opcode %d\n",
 				first_ce->myaiocb.aio_fildes,
-				first_ce->myaiocb.aio_offset,
-				(long)(first_ce->myaiocb.aio_buf),
+				(long long)first_ce->myaiocb.aio_offset,
+				first_ce->myaiocb.aio_buf,
 				first_ce->myaiocb.aio_nbytes,
 				first_ce->myaiocb.aio_lio_opcode
-#else
-			printf("aio_return error2: fd %d offset %lld buffer %lx size %d Opcode %d\n",
-				first_ce->myaiocb64.aio_fildes,
-				first_ce->myaiocb64.aio_offset,
-				(long)(first_ce->myaiocb64.aio_buf),
-				first_ce->myaiocb64.aio_nbytes,
-				first_ce->myaiocb64.aio_lio_opcode
-#endif
-#else
-			printf("aio_return error2: fd %d offset %d buffer %lx size %d Opcode %d\n",
-				first_ce->myaiocb.aio_fildes,
-				first_ce->myaiocb.aio_offset,
-				(long)(first_ce->myaiocb.aio_buf),
-				first_ce->myaiocb.aio_nbytes,
-				first_ce->myaiocb.aio_lio_opcode
-#endif
 				);
 		}
 		if(retval > 0)
 		{
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 			mbcopy((char *)first_ce->myaiocb.aio_buf,(char *)ubuffer,(size_t)retval);
-#else
-			mbcopy((char *)first_ce->myaiocb64.aio_buf,(char *)ubuffer,(size_t)retval);
-#endif
-#else
-			mbcopy((char *)first_ce->myaiocb.aio_buf,(char *)ubuffer,(size_t)retval);
-#endif
 		}
 		first_ce->direct=0;
 		takeoff_cache(gc,first_ce);
@@ -594,14 +513,18 @@ out:
  * all memory associated with this cache entry.
  ************************************************************************/
 
+#ifdef HAVE_ANSIC_C
+struct cache_ent * alloc_cache(struct cache *gc,long long fd,off64_t offset,long long size,long long op)
+#else
 struct cache_ent *
 alloc_cache(gc,fd,offset,size,op)
 struct cache *gc;
 long long fd,size,op;
 off64_t offset;
+#endif
 {
 	struct cache_ent *ce;
-	long temp;
+	intptr_t temp;
 	ce=(struct cache_ent *)malloc((size_t)sizeof(struct cache_ent));
 	if(ce == (struct cache_ent *)0)
 	{
@@ -609,56 +532,23 @@ off64_t offset;
 		exit(175);
 	}
 	bzero(ce,sizeof(struct cache_ent));
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 	ce->myaiocb.aio_fildes=(int)fd;
 	ce->myaiocb.aio_offset=(off64_t)offset;
-	ce->real_address = (char *)malloc((size_t)(size+page_size));
-	temp=(long)ce->real_address;
-	temp = (temp+page_size) & ~(page_size-1);
+	ce->real_address = malloc((size_t)(size+page_size));
+printf("\nAllocate buffer2 %p Size %lld \n",ce->real_address,size+page_size);
+	temp = (intptr_t)ce->real_address;
+	temp = (temp+(page_size-1)) & ~(page_size-1);
 	ce->myaiocb.aio_buf=(volatile void *)temp;
-	if(ce->myaiocb.aio_buf == 0)
-#else
-	ce->myaiocb64.aio_fildes=(int)fd;
-	ce->myaiocb64.aio_offset=(off64_t)offset;
-	ce->real_address = (char *)malloc((size_t)(size+page_size));
-	temp=(long)ce->real_address;
-	temp = (temp+page_size) & ~(page_size-1);
-	ce->myaiocb64.aio_buf=(volatile void *)temp;
-	if(ce->myaiocb64.aio_buf == 0)
-#endif
-#else
-	ce->myaiocb.aio_fildes=(int)fd;
-	ce->myaiocb.aio_offset=(off_t)offset;
-	ce->real_address = (char *)malloc((size_t)(size+page_size));
-	temp=(long)ce->real_address;
-	temp = (temp+page_size) & ~(page_size-1);
-	ce->myaiocb.aio_buf=(volatile void *)temp;
-	if(ce->myaiocb.aio_buf == 0)
-#endif
+	if(ce->myaiocb.aio_buf == NULL)
 	{
 		printf("Malloc failed\n");
 		exit(176);
 	}
 	/*bzero(ce->myaiocb.aio_buf,(size_t)size);*/
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 	ce->myaiocb.aio_reqprio=0;
 	ce->myaiocb.aio_nbytes=(size_t)size;
 	ce->myaiocb.aio_sigevent.sigev_notify=SIGEV_NONE;
 	ce->myaiocb.aio_lio_opcode=(int)op;
-#else
-	ce->myaiocb64.aio_reqprio=0;
-	ce->myaiocb64.aio_nbytes=(size_t)size;
-	ce->myaiocb64.aio_sigevent.sigev_notify=SIGEV_NONE;
-	ce->myaiocb64.aio_lio_opcode=(int)op;
-#endif
-#else
-	ce->myaiocb.aio_reqprio=0;
-	ce->myaiocb.aio_nbytes=(size_t)size;
-	ce->myaiocb.aio_sigevent.sigev_notify=SIGEV_NONE;
-	ce->myaiocb.aio_lio_opcode=(int)op;
-#endif
 	ce->fd=(int)fd;
 	ce->forward=0;
 	ce->back=gc->tail;
@@ -675,11 +565,16 @@ off64_t offset;
  * This routine checks to see if the requested data is in the
  * cache. 
 *************************************************************************/
+#ifdef HAVE_ANSIC_C
+struct cache_ent *
+incache(struct cache *gc, long long fd, off64_t offset, long long size)
+#else
 struct cache_ent *
 incache(gc,fd,offset,size)
 struct cache *gc;
 long long fd,size;
 off64_t offset;
+#endif
 {
 	struct cache_ent *move;
 	if(gc->head==0)
@@ -687,8 +582,6 @@ off64_t offset;
 		return(0);
 	}
 	move=gc->head;
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 	while(move)
 	{
 		if((move->fd == fd) && (move->myaiocb.aio_offset==(off64_t)offset) &&
@@ -698,28 +591,6 @@ off64_t offset;
 			}
 		move=move->forward;
 	}
-#else
-	while(move)
-	{
-		if((move->fd == fd) && (move->myaiocb64.aio_offset==(off64_t)offset) &&
-			((size_t)size==move->myaiocb64.aio_nbytes))
-			{
-				return(move);
-			}
-		move=move->forward;
-	}
-#endif
-#else
-	while(move)
-	{
-		if((move->fd == fd) && (move->myaiocb.aio_offset==(off_t)offset) &&
-			((size_t)size==move->myaiocb.aio_nbytes))
-			{
-				return(move);
-			}
-		move=move->forward;
-	}
-#endif
 	return(0);
 }
 
@@ -729,9 +600,7 @@ off64_t offset;
 *************************************************************************/
 
 void
-takeoff_cache(gc,ce)
-struct cache *gc;
-struct cache_ent *ce;
+takeoff_cache(struct cache *gc, struct cache_ent *ce)
 {
 	struct cache_ent *move;
 	long long found;
@@ -746,8 +615,12 @@ struct cache_ent *ce;
 			gc->tail = 0;
 		if(!ce->direct)
 		{
-			free((void *)(ce->real_address));
-			free((void *)ce);
+			if(ce->real_address != NULL)
+			   free((void *)(ce->real_address));
+			ce->real_address = NULL;
+			if(ce != NULL)
+			   free((void *)ce);
+			ce = NULL;
 		}
 		gc->count--;
 		return;
@@ -780,8 +653,12 @@ struct cache_ent *ce;
 	move=gc->head;
 	if(!ce->direct)
 	{
-		free((void *)(ce->real_address));
-		free((void *)ce);
+		if(ce->real_address != NULL)
+		   free((void *)(ce->real_address));
+		ce->real_address = NULL;
+		if(ce != NULL)
+		   free((void *)ce);
+		ce = NULL;
 	}
 	gc->count--;
 }
@@ -792,9 +669,14 @@ struct cache_ent *ce;
  * be satisfied from the cache. This indicates that the previous
  * async read-ahead was not correct and a new pattern is emerging. 
  ************************************************************************/
+#ifdef HAVE_ANSIC_C
+void
+del_cache(struct cache *gc)
+#else
 void
 del_cache(gc)
 struct cache *gc;
+#endif
 {
 	struct cache_ent *ce;
 	ssize_t ret;
@@ -804,26 +686,10 @@ struct cache *gc;
 		ce=gc->head;
 		if(ce==0)
 			return;
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		while((ret = aio_cancel(0,&ce->myaiocb))==AIO_NOTCANCELED)
-#else
-		while((ret = aio_cancel64(0,&ce->myaiocb64))==AIO_NOTCANCELED)
-#endif
-#else
-		while((ret = aio_cancel(0,&ce->myaiocb))==AIO_NOTCANCELED)
-#endif
 			; 
 
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		ret = aio_return(&ce->myaiocb);
-#else
-		ret = aio_return64(&ce->myaiocb64);
-#endif
-#else
-		ret = aio_return(&ce->myaiocb);
-#endif
 		ce->direct=0;
 		takeoff_cache(gc,ce);	  /* remove from cache */
 	}
@@ -837,6 +703,10 @@ struct cache *gc;
  * async I/O to be performed and does not require any bcopy to be
  * done to put the data back into the location specified by the caller.
  ************************************************************************/
+#ifdef HAVE_ANSIC_C
+int
+async_read_no_copy(struct cache *gc, long long fd, char **ubuffer, off64_t offset, long long size, long long stride, off64_t max, long long depth)
+#else
 int
 async_read_no_copy(gc, fd, ubuffer, offset, size, stride, max, depth)
 struct cache *gc;
@@ -847,6 +717,7 @@ long long size;
 long long stride;
 off64_t max;
 long long depth;
+#endif
 {
 	off64_t a_offset,r_offset;
 	long long a_size;
@@ -864,100 +735,29 @@ long long depth;
 	 */
 	if((ce=(struct cache_ent *)incache(gc,fd,offset,size)))
 	{
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
 		{
 			async_suspend(ce);
 		}
-#else
-		while((ret=aio_error64(&ce->myaiocb64))== EINPROGRESS)
-		{
-			async_suspend(ce);
-		}
-#endif
-#else
-		while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
-		{
-			async_suspend(ce);
-		}
-#endif
 		if(ret)
-			printf("aio_error 3: ret %d %d\n",ret,errno);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-		if(ce->oldbuf != ce->myaiocb.aio_buf ||
-			ce->oldfd != ce->myaiocb.aio_fildes ||
-			ce->oldsize != ce->myaiocb.aio_nbytes) 
-#else
-		if(ce->oldbuf != ce->myaiocb64.aio_buf ||
-			ce->oldfd != ce->myaiocb64.aio_fildes ||
-			ce->oldsize != ce->myaiocb64.aio_nbytes) 
-#endif
-#else
-		if(ce->oldbuf != ce->myaiocb.aio_buf ||
-			ce->oldfd != ce->myaiocb.aio_fildes ||
-			ce->oldsize != ce->myaiocb.aio_nbytes) 
-#endif
+			printf("aio_error 3: ret %zd %d\n",ret,errno);
 			printf("It changed in flight\n");
 			
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		retval=aio_return(&ce->myaiocb);
-#else
-		retval=aio_return64(&ce->myaiocb64);
-#endif
-#else
-		retval=aio_return(&ce->myaiocb);
-#endif
 		if(retval > 0)
 		{
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-			*ubuffer=(char *)ce->myaiocb.aio_buf;
-#else
-			*ubuffer=(char *)ce->myaiocb64.aio_buf;
-#endif
-#else
-			*ubuffer=(char *)ce->myaiocb.aio_buf;
-#endif
+			*ubuffer= (char *)ce->myaiocb.aio_buf;
 		}else
-			*ubuffer=0;
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
+			*ubuffer= NULL;
 		if(retval < ce->myaiocb.aio_nbytes)
-#else
-		if(retval < ce->myaiocb64.aio_nbytes)
-#endif
-#else
-		if(retval < ce->myaiocb.aio_nbytes)
-#endif
 		{
-			printf("aio_return error4: ret %d %d\n",retval,errno);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-			printf("aio_return error4: fd %d offset %lld buffer %lx size %d Opcode %d\n",
+			printf("aio_return error4: ret %zd %d\n",retval,errno);
+			printf("aio_return error4: fd %d offset %lld buffer %p size %zd Opcode %d\n",
 				ce->myaiocb.aio_fildes,
-				ce->myaiocb.aio_offset,
-				(long)(ce->myaiocb.aio_buf),
+				(long long)ce->myaiocb.aio_offset,
+				ce->myaiocb.aio_buf,
 				ce->myaiocb.aio_nbytes,
 				ce->myaiocb.aio_lio_opcode
-#else
-			printf("aio_return error4: fd %d offset %lld buffer %lx size %d Opcode %d\n",
-				ce->myaiocb64.aio_fildes,
-				ce->myaiocb64.aio_offset,
-				(long)(ce->myaiocb64.aio_buf),
-				ce->myaiocb64.aio_nbytes,
-				ce->myaiocb64.aio_lio_opcode
-#endif
-#else
-			printf("aio_return error4: fd %d offset %d buffer %lx size %d Opcode %d\n",
-				ce->myaiocb.aio_fildes,
-				ce->myaiocb.aio_offset,
-				(long)(ce->myaiocb.aio_buf),
-				ce->myaiocb.aio_nbytes,
-				ce->myaiocb.aio_lio_opcode
-#endif
 				);
 		}
 		ce->direct=1;
@@ -973,30 +773,16 @@ long long depth;
 		first_ce=alloc_cache(gc,fd,offset,size,(long long)LIO_READ); /* allocate buffer */
 		/*printf("allocated buffer/read %x offset %d\n",first_ce->myaiocb.aio_buf,offset);*/
 again:
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		first_ce->oldbuf=first_ce->myaiocb.aio_buf;
 		first_ce->oldfd=first_ce->myaiocb.aio_fildes;
 		first_ce->oldsize=first_ce->myaiocb.aio_nbytes;
 		ret=aio_read(&first_ce->myaiocb);
-#else
-		first_ce->oldbuf=first_ce->myaiocb64.aio_buf;
-		first_ce->oldfd=first_ce->myaiocb64.aio_fildes;
-		first_ce->oldsize=first_ce->myaiocb64.aio_nbytes;
-		ret=aio_read64(&first_ce->myaiocb64);
-#endif
-#else
-		first_ce->oldbuf=first_ce->myaiocb.aio_buf;
-		first_ce->oldfd=first_ce->myaiocb.aio_fildes;
-		first_ce->oldsize=first_ce->myaiocb.aio_nbytes;
-		ret=aio_read(&first_ce->myaiocb);
-#endif
 		if(ret!=0)
 		{
 			if(errno==EAGAIN)
 				goto again;
 			else
-				printf("error returned from aio_read(). Ret %d errno %d\n",ret,errno);
+				printf("error returned from aio_read(). Ret %zd errno %d\n",ret,errno);
 		}
 	}
 	if(stride==0)	 /* User does not want read-ahead */
@@ -1021,24 +807,10 @@ again:
 		if((ce=incache(gc,fd,r_offset,a_size)))
 			continue;
 		ce=alloc_cache(gc,fd,r_offset,a_size,(long long)LIO_READ);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		ce->oldbuf=ce->myaiocb.aio_buf;
 		ce->oldfd=ce->myaiocb.aio_fildes;
 		ce->oldsize=ce->myaiocb.aio_nbytes;
 		ret=aio_read(&ce->myaiocb);
-#else
-		ce->oldbuf=ce->myaiocb64.aio_buf;
-		ce->oldfd=ce->myaiocb64.aio_fildes;
-		ce->oldsize=ce->myaiocb64.aio_nbytes;
-		ret=aio_read64(&ce->myaiocb64);
-#endif
-#else
-		ce->oldbuf=ce->myaiocb.aio_buf;
-		ce->oldfd=ce->myaiocb.aio_fildes;
-		ce->oldsize=ce->myaiocb.aio_nbytes;
-		ret=aio_read(&ce->myaiocb);
-#endif
 		if(ret!=0)
 		{
 			takeoff_cache(gc,ce);
@@ -1048,97 +820,33 @@ again:
 out:
 	if(del_read)	/* Wait for the first read to complete */
 	{
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		while((ret=aio_error(&first_ce->myaiocb))== EINPROGRESS)
 		{
 			async_suspend(first_ce);
 		}
-#else
-		while((ret=aio_error64(&first_ce->myaiocb64))== EINPROGRESS)
-		{
-			async_suspend(first_ce);
-		}
-#endif
-#else
-		while((ret=aio_error(&first_ce->myaiocb))== EINPROGRESS)
-		{
-			async_suspend(first_ce);
-		}
-#endif
 		if(ret)
-			printf("aio_error 4: ret %d %d\n",ret,errno);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
+			printf("aio_error 4: ret %zd %d\n",ret,errno);
 		if(first_ce->oldbuf != first_ce->myaiocb.aio_buf ||
 			first_ce->oldfd != first_ce->myaiocb.aio_fildes ||
 			first_ce->oldsize != first_ce->myaiocb.aio_nbytes) 
 			printf("It changed in flight2\n");
 		retval=aio_return(&first_ce->myaiocb);
-#else
-		if(first_ce->oldbuf != first_ce->myaiocb64.aio_buf ||
-			first_ce->oldfd != first_ce->myaiocb64.aio_fildes ||
-			first_ce->oldsize != first_ce->myaiocb64.aio_nbytes) 
-			printf("It changed in flight2\n");
-		retval=aio_return64(&first_ce->myaiocb64);
-#endif
-#else
-		if(first_ce->oldbuf != first_ce->myaiocb.aio_buf ||
-			first_ce->oldfd != first_ce->myaiocb.aio_fildes ||
-			first_ce->oldsize != first_ce->myaiocb.aio_nbytes) 
-			printf("It changed in flight2\n");
-		retval=aio_return(&first_ce->myaiocb);
-#endif
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 		if(retval < first_ce->myaiocb.aio_nbytes)
-#else
-		if(retval < first_ce->myaiocb64.aio_nbytes)
-#endif
-#else
-		if(retval < first_ce->myaiocb.aio_nbytes)
-#endif
 		{
-			printf("aio_return error5: ret %d %d\n",retval,errno);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-			printf("aio_return error5: fd %d offset %lld buffer %lx size %d Opcode %d\n",
+			printf("aio_return error5: ret %zd %d\n",retval,errno);
+			printf("aio_return error5: fd %d offset %lld buffer %p size %zd Opcode %d\n",
 				first_ce->myaiocb.aio_fildes,
-				first_ce->myaiocb.aio_offset,
-				(long)(first_ce->myaiocb.aio_buf),
+				(long long)first_ce->myaiocb.aio_offset,
+				first_ce->myaiocb.aio_buf,
 				first_ce->myaiocb.aio_nbytes,
 				first_ce->myaiocb.aio_lio_opcode
-#else
-			printf("aio_return error5: fd %d offset %lld buffer %lx size %d Opcode %d\n",
-				first_ce->myaiocb64.aio_fildes,
-				first_ce->myaiocb64.aio_offset,
-				(long)(first_ce->myaiocb64.aio_buf),
-				first_ce->myaiocb64.aio_nbytes,
-				first_ce->myaiocb64.aio_lio_opcode
-#endif
-#else
-			printf("aio_return error5: fd %d offset %ld buffer %lx size %d Opcode %d\n",
-				first_ce->myaiocb.aio_fildes,
-				first_ce->myaiocb.aio_offset,
-				(long)(first_ce->myaiocb.aio_buf),
-				first_ce->myaiocb.aio_nbytes,
-				first_ce->myaiocb.aio_lio_opcode
-#endif
 				);
 		}
 		if(retval > 0)
 		{
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-			*ubuffer=(char *)first_ce->myaiocb.aio_buf;
-#else
-			*ubuffer=(char *)first_ce->myaiocb64.aio_buf;
-#endif
-#else
-			*ubuffer=(char *)first_ce->myaiocb.aio_buf;
-#endif
+			*ubuffer= (char *)first_ce->myaiocb.aio_buf;
 		}else
-			*ubuffer=(char *)0;
+			*ubuffer= NULL;
 		first_ce->direct=1;	 /* do not delete the buffer */
 		takeoff_cache(gc,first_ce);
 		putoninuse(gc,first_ce);
@@ -1151,9 +859,13 @@ out:
  * the library is now free to return the memory to the pool for later
  * reuse.
  ************************************************************************/
+#ifdef HAVE_ANSIC_C
+void async_release(struct cache *gc)
+#else
 void
 async_release(gc)
 struct cache *gc;
+#endif
 {
 	takeoffinuse(gc);
 }
@@ -1164,10 +876,15 @@ struct cache *gc;
  * the buffer it will call back into async_release and the items on the 
  * inuse list will be deallocated.
  ************************************************************************/
+#ifdef HAVE_ANSIC_C
+void
+putoninuse(struct cache *gc,struct cache_ent *entry)
+#else
 void
 putoninuse(gc,entry)
 struct cache *gc;
 struct cache_ent *entry;
+#endif
 {
 	if(gc->inuse_head)
 		entry->forward=gc->inuse_head;
@@ -1180,9 +897,14 @@ struct cache_ent *entry;
  * This is called when the application is finished with the data that
  * was provided. The memory may now be returned to the pool.
  ************************************************************************/
+#ifdef HAVE_ANSIC_C
+void
+takeoffinuse(struct cache *gc)
+#else
 void
 takeoffinuse(gc)
 struct cache *gc;
+#endif
 {
 	struct cache_ent *ce;
 	if(gc->inuse_head==0)
@@ -1192,8 +914,12 @@ struct cache *gc;
 	
 	if(gc->inuse_head !=0)
 		printf("Error in take off inuse\n");
-	free((void*)(ce->real_address));
-	free(ce);
+	if(ce->real_address != NULL)
+	   free((void*)(ce->real_address));
+	ce->real_address = NULL;
+	if(ce != NULL)
+	   free(ce);
+	ce = NULL;
 }
 
 /*************************************************************************
@@ -1205,6 +931,10 @@ struct cache *gc;
  * depth  ..... How much read-ahead do you want.
  * 
  *************************************************************************/
+#ifdef HAVE_ANSIC_C
+size_t
+async_write(struct cache *gc,long long fd,char *buffer,long long size,off64_t offset,long long depth)
+#else
 size_t
 async_write(gc,fd,buffer,size,offset,depth)
 struct cache *gc;
@@ -1212,37 +942,22 @@ long long fd,size;
 char *buffer;
 off64_t offset;
 long long depth;
+#endif
 {
 	struct cache_ent *ce;
 	size_t ret;
 	ce=allocate_write_buffer(gc,fd,offset,size,(long long)LIO_WRITE,depth,0LL,(char *)0,(char *)0);
 	ce->direct=0;	 /* not direct. Lib supplies buffer and must free it */
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 	mbcopy(buffer,(char *)(ce->myaiocb.aio_buf),(size_t)size);
-#else
-	mbcopy(buffer,(char *)(ce->myaiocb64.aio_buf),(size_t)size);
-#endif
-#else
-	mbcopy(buffer,(char *)(ce->myaiocb.aio_buf),(size_t)size);
-#endif
 	async_put_on_write_queue(gc,ce);
 	/*
-	printf("asw: fd %d offset %lld, size %d\n",ce->myaiocb64.aio_fildes,
-		ce->myaiocb64.aio_offset,
-		ce->myaiocb64.aio_nbytes);
+	printf("asw: fd %d offset %lld, size %zd\n",ce->myaiocb.aio_fildes,
+		ce->myaiocb.aio_offset,
+		ce->myaiocb.aio_nbytes);
 	*/	
 
 again:
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 	ret=aio_write(&ce->myaiocb);
-#else
-	ret=aio_write64(&ce->myaiocb64);
-#endif
-#else
-	ret=aio_write(&ce->myaiocb);
-#endif
 	if(ret==-1)
 	{
 		if(errno==EAGAIN)
@@ -1258,13 +973,13 @@ again:
 		}
 		else
 		{
-			printf("Error in aio_write: ret %d errno %d count %lld\n",ret,errno,gc->w_count);
+			printf("Error in aio_write: ret %zd errno %d count %lld\n",ret,errno,gc->w_count);
 			/*
-			printf("aio_write_no_copy: fd %d buffer %x offset %lld size %d\n",
-				ce->myaiocb64.aio_fildes,
-				ce->myaiocb64.aio_buf,
-				ce->myaiocb64.aio_offset,
-				ce->myaiocb64.aio_nbytes);
+			printf("aio_write_no_copy: fd %d buffer %x offset %lld size %zd\n",
+				ce->myaiocb.aio_fildes,
+				ce->myaiocb.aio_buf,
+				ce->myaiocb.aio_offset,
+				ce->myaiocb.aio_nbytes);
 			*/
 			exit(177);
 		}
@@ -1278,6 +993,11 @@ again:
  * needed.
  *************************************************************************/
 
+#ifdef HAVE_ANSIC_C
+struct cache_ent *
+allocate_write_buffer( struct cache *gc, long long fd, long long offset, long long size,long long op, 
+	long long w_depth, long long direct, char *buffer, char *free_addr)
+#else
 struct cache_ent *
 allocate_write_buffer(gc,fd,offset,size,op,w_depth,direct,buffer,free_addr)
 struct cache *gc;
@@ -1286,9 +1006,10 @@ off64_t offset;
 long long w_depth;
 long long direct;
 char *buffer,*free_addr;
+#endif
 {
 	struct cache_ent *ce;
-	long temp;
+	intptr_t temp;
 	if(fd==0LL)
 	{
 		printf("Setting up write buffer insane\n");
@@ -1303,47 +1024,13 @@ char *buffer,*free_addr;
 		exit(179);
 	}
 	bzero(ce,sizeof(struct cache_ent));
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-	ce->myaiocb.aio_fildes=(int)fd;
-	ce->myaiocb.aio_offset=(off64_t)offset;
-	if(!direct)
-	{
-		ce->real_address = (char *)malloc((size_t)(size+page_size));
-		temp=(long)ce->real_address;
-		temp = (temp+page_size) & ~(page_size-1);
-		ce->myaiocb.aio_buf=(volatile void *)temp;
-	}else
-	{
-		ce->myaiocb.aio_buf=(volatile void *)buffer;
-		ce->real_address=(char *)free_addr;
-	}
-	if(ce->myaiocb.aio_buf == 0)
-#else
-	ce->myaiocb64.aio_fildes=(int)fd;
-	ce->myaiocb64.aio_offset=(off64_t)offset;
-	if(!direct)
-	{
-		ce->real_address = (char *)malloc((size_t)(size+page_size));
-		temp=(long)ce->real_address;
-		temp = (temp+page_size) & ~(page_size-1);
-		ce->myaiocb64.aio_buf=(volatile void *)temp;
-	}
-	else
-	{
-		ce->myaiocb64.aio_buf=(volatile void *)buffer;
-		ce->real_address=(char *)free_addr;
-	}
-	if(ce->myaiocb64.aio_buf == 0)
-#endif
-#else
 	ce->myaiocb.aio_fildes=(int)fd;
 	ce->myaiocb.aio_offset=(off_t)offset;
 	if(!direct)
 	{
-		ce->real_address = (char *)malloc((size_t)(size+page_size));
-		temp=(long)ce->real_address;
-		temp = (temp+page_size) & ~(page_size-1);
+		ce->real_address = malloc((size_t)(size+page_size));
+		temp = (intptr_t)ce->real_address;
+		temp = (temp+(page_size-1)) & ~(page_size-1);
 		ce->myaiocb.aio_buf=(volatile void *)temp;
 	}
 	else
@@ -1352,29 +1039,14 @@ char *buffer,*free_addr;
 		ce->real_address=(char *)free_addr;
 	}
 	if(ce->myaiocb.aio_buf == 0)
-#endif
 	{
 		printf("Malloc failed 2\n");
 		exit(180);
 	}
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 	ce->myaiocb.aio_reqprio=0;
 	ce->myaiocb.aio_nbytes=(size_t)size;
 	ce->myaiocb.aio_sigevent.sigev_notify=SIGEV_NONE;
 	ce->myaiocb.aio_lio_opcode=(int)op;
-#else
-	ce->myaiocb64.aio_reqprio=0;
-	ce->myaiocb64.aio_nbytes=(size_t)size;
-	ce->myaiocb64.aio_sigevent.sigev_notify=SIGEV_NONE;
-	ce->myaiocb64.aio_lio_opcode=(int)op;
-#endif
-#else
-	ce->myaiocb.aio_reqprio=0;
-	ce->myaiocb.aio_nbytes=(size_t)size;
-	ce->myaiocb.aio_sigevent.sigev_notify=SIGEV_NONE;
-	ce->myaiocb.aio_lio_opcode=(int)op;
-#endif
 	ce->fd=(int)fd;
 	return(ce);
 }
@@ -1383,10 +1055,15 @@ char *buffer,*free_addr;
  * Put it on the outbound queue.
  *************************************************************************/
 
+#ifdef HAVE_ANSIC_C
+void
+async_put_on_write_queue(struct cache *gc,struct cache_ent *ce)
+#else
 void
 async_put_on_write_queue(gc,ce)
 struct cache *gc;
 struct cache_ent *ce;
+#endif
 {
 	ce->forward=0;
 	ce->back=gc->w_tail;
@@ -1402,13 +1079,17 @@ struct cache_ent *ce;
 /*************************************************************************
  * Cleanup all outstanding writes
  *************************************************************************/
+#ifdef HAVE_AHSIC_C
+void
+async_write_finish(struct cache *gc)
+#else
 void
 async_write_finish(gc)
 struct cache *gc;
+#endif
 {
 	while(gc->w_head)
 	{
-		/*printf("async_write_finish: Waiting for buffer %x to finish\n",gc->w_head->myaiocb64.aio_buf);*/
 		async_wait_for_write(gc);
 	}
 }
@@ -1417,92 +1098,62 @@ struct cache *gc;
  * Wait for an I/O to finish
  *************************************************************************/
 
+#ifdef HAVE_ANSIC_C
+void
+async_wait_for_write(struct cache *gc)
+#else
 void
 async_wait_for_write(gc)
 struct cache *gc;
+#endif
 {
 	struct cache_ent *ce;
-	size_t ret,retval;
+	size_t ret;
+	int retval;
 	if(gc->w_head==0)
 		return;
 	ce=gc->w_head;
+        if(ce == NULL)
+		return;
 	gc->w_head=ce->forward;
 	gc->w_count--;
-	ce->forward=0;
+	ce->forward=NULL;
 	if(ce==gc->w_tail)
 		gc->w_tail=0;
-	/*printf("Wait for buffer %x  offset %lld  size %d to finish\n",
-		ce->myaiocb64.aio_buf,
-		ce->myaiocb64.aio_offset,
-		ce->myaiocb64.aio_nbytes);
+	/*printf("Wait for buffer %x  offset %lld  size %zd to finish\n",
+		ce->myaiocb.aio_buf,
+		ce->myaiocb.aio_offset,
+		ce->myaiocb.aio_nbytes);
 	printf("write count %lld \n",gc->w_count);
 	*/
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 	while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
 	{
 		async_suspend(ce);
 	}
-#else
-	while((ret=aio_error64(&ce->myaiocb64))== EINPROGRESS)
-	{
-		async_suspend(ce);
-	}
-#endif
-#else
-	while((ret=aio_error(&ce->myaiocb))== EINPROGRESS)
-	{
-		async_suspend(ce);
-	}
-#endif
 	if(ret)
 	{
-		printf("aio_error 5: ret %d %d\n",ret,errno);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-		printf("fd %d offset %lld size %d\n",
+		printf("aio_error 5: ret %zd %d\n",ret,errno);
+		printf("fd %d offset %lld size %zd\n",
 			ce->myaiocb.aio_fildes,
-			ce->myaiocb.aio_offset,
+			(long long)ce->myaiocb.aio_offset,
 			ce->myaiocb.aio_nbytes);
-#else
-		printf("fd %d offset %lld size %d\n",
-			ce->myaiocb64.aio_fildes,
-			ce->myaiocb64.aio_offset,
-			ce->myaiocb64.aio_nbytes);
-#endif
-#else
-		printf("fd %d offset %lld size %d\n",
-			ce->myaiocb.aio_fildes,
-			ce->myaiocb.aio_offset,
-			ce->myaiocb.aio_nbytes);
-#endif
 		exit(181);
 	}
 
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 	retval=aio_return(&ce->myaiocb);
-#else
-#if defined(__CrayX1__)
-	retval=aio_return64((aiocb64_t *)&ce->myaiocb64);
-#else
-	retval=aio_return64((struct aiocb64 *)&ce->myaiocb64);
-#endif
-
-#endif
-#else
-	retval=aio_return(&ce->myaiocb);
-#endif
-	if((int)retval < 0)
+	if(retval < 0)
 	{
 		printf("aio_return error: %d\n",errno);
 	}
 
 	if(!ce->direct)
 	{
-		/* printf("Freeing buffer %x\n",ce->real_address);*/
-		free((void *)(ce->real_address));
-		free((void *)ce);
+		if(ce->real_address != NULL)
+		   free((void *)(ce->real_address)); /* Causes crash. */
+		ce->real_address=NULL;
+		if(ce != NULL)
+		   free((void *)ce);
+		ce=NULL;
 	}
 
 }
@@ -1517,14 +1168,19 @@ struct cache *gc;
  * free_addr .. address of memory to free after write is completed.
  * 
  *************************************************************************/
+#ifdef HAVE_ANSIC_C
+size_t
+async_write_no_copy(struct cache *gc,long long fd,char *buffer,long long size,long long offset,long long depth,char *free_addr)
+#else
 size_t
 async_write_no_copy(gc,fd,buffer,size,offset,depth,free_addr)
 struct cache *gc;
 long long fd,size;
 char *buffer;
-off64_t offset;
+long long offset;
 long long depth;
 char *free_addr;
+#endif
 {
 	struct cache_ent *ce;
 	size_t ret;
@@ -1533,21 +1189,13 @@ char *free_addr;
 	ce->direct=0;	/* have library de-allocate the buffer */
 	async_put_on_write_queue(gc,ce);
 	/*
-	printf("awnc: fd %d offset %lld, size %d\n",ce->myaiocb64.aio_fildes,
-		ce->myaiocb64.aio_offset,
-		ce->myaiocb64.aio_nbytes);
+	printf("awnc: fd %d offset %lld, size %zd\n",ce->myaiocb.aio_fildes,
+		ce->myaiocb.aio_offset,
+		ce->myaiocb.aio_nbytes);
 	*/
 
 again:
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
 	ret=aio_write(&ce->myaiocb);
-#else
-	ret=aio_write64(&ce->myaiocb64);
-#endif
-#else
-	ret=aio_write(&ce->myaiocb);
-#endif
 	if(ret==-1)
 	{
 		if(errno==EAGAIN)
@@ -1563,28 +1211,12 @@ again:
 		}
 		else
 		{
-			printf("Error in aio_write: ret %d errno %d\n",ret,errno);
-#ifdef _LARGEFILE64_SOURCE 
-#ifdef __LP64__
-			printf("aio_write_no_copy: fd %d buffer %lx offset %lld size %d\n",
+			printf("Error in aio_write: ret %zd errno %d\n",ret,errno);
+			printf("aio_write_no_copy: fd %d buffer %p offset %lld size %zd\n",
 				ce->myaiocb.aio_fildes,
-				(long)(ce->myaiocb.aio_buf),
-				ce->myaiocb.aio_offset,
+				ce->myaiocb.aio_buf,
+				(long long)ce->myaiocb.aio_offset,
 				ce->myaiocb.aio_nbytes);
-#else
-			printf("aio_write_no_copy: fd %d buffer %lx offset %lld size %d\n",
-				ce->myaiocb64.aio_fildes,
-				(long)(ce->myaiocb64.aio_buf),
-				ce->myaiocb64.aio_offset,
-				ce->myaiocb64.aio_nbytes);
-#endif
-#else
-			printf("aio_write_no_copy: fd %d buffer %lx offset %ld size %d\n",
-				ce->myaiocb.aio_fildes,
-				(long)(ce->myaiocb.aio_buf),
-				ce->myaiocb.aio_offset,
-				ce->myaiocb.aio_nbytes);
-#endif
 			exit(182);
 		}
 	} 
@@ -1595,10 +1227,12 @@ again:
 }
 
 void mbcopy(source, dest, len)
-char *source,*dest;
+const char *source;
+char *dest;
 size_t len;
 {
 	int i;
 	for(i=0;i<len;i++)
 		*dest++=*source++;
 }
+
